import os
import time
from typing import List, Optional

import click

from vessl.cli._base import VesslGroup, vessl_argument, vessl_option
from vessl.cli._util import (
    print_data,
    print_logs,
    print_table,
    print_volume_files,
    truncate_datetime,
)
from vessl.util.prompt import prompt_choices, generic_prompter
from vessl.util.echo import print_info, print_success
from vessl.util.endpoint import Endpoint
from vessl.cli.kernel_cluster import cluster_option
from vessl.cli.kernel_resource_spec import resource_option
from vessl.cli.organization import organization_name_option
from vessl.cli.project import project_name_option
from vessl.experiment import (
    create_experiment,
    delete_experiment,
    download_experiment_output_files,
    list_experiment_logs,
    list_experiment_output_files,
    list_experiments,
    read_experiment,
    terminate_experiment,
)
from vessl.kernel_cluster import list_cluster_nodes
from vessl.kernel_image import list_kernel_images
from vessl.organization import list_organization_credentials
from vessl.util import logger
from vessl.util.constant import (
    EXPERIMENT_WORKING_DIR,
    FRAMEWORK_TYPE_PYTORCH,
    FRAMEWORK_TYPE_TENSORFLOW,
    FRAMEWORK_TYPES,
    MOUNT_PATH_OUTPUT,
    PROCESSOR_TYPE_GPU,
    PROCESSOR_TYPES,
)


def experiment_number_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    experiments = list_experiments()
    return prompt_choices("Experiment", [(f"#{x.number}", x.number) for x in reversed(experiments)])


def local_experiment_number_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    experiments = [e for e in list_experiments() if e.is_local]
    return prompt_choices("Experiment", [(f"#{x.number}", x.number) for x in reversed(experiments)])


def processor_type_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before processor type (`--processor-type`).",
        )

    if ctx.obj.get("resource") is None:
        processor_type = prompt_choices("Processor Type", PROCESSOR_TYPES)
        ctx.obj["processor_type"] = processor_type
        return processor_type


def processor_type_callback(ctx: click.Context, param: click.Parameter, value: str):
    if value:
        ctx.obj["processor_type"] = value
    return value


def cpu_limit_prompter(ctx: click.Context, param: click.Parameter, value: float) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before CPU limit (`--cpu-limit`).",
        )

    if ctx.obj.get("resource") is None:
        return click.prompt("CPUs (in vCPU)", type=click.FLOAT)


def memory_limit_prompter(ctx: click.Context, param: click.Parameter, value: float) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before memory limit (`--memory-limit`).",
        )

    if ctx.obj.get("resource") is None:
        return click.prompt("Memory (e.g. 4Gi)", type=click.STRING)


def gpu_type_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before GPU type (`--gpu-type`).",
        )

    if ctx.obj.get("resource") is None:
        processor_type = ctx.obj.get("processor_type")
        if processor_type is None:
            raise click.UsageError(
                message="Processor type must be specified before GPU type (`--gpu-type`).",
            )
        if processor_type == PROCESSOR_TYPE_GPU:
            nodes = list_cluster_nodes(cluster.id)
            return prompt_choices("GPU Type", [x.gpu_product_name for x in nodes])


def gpu_limit_prompter(ctx: click.Context, param: click.Parameter, value: float) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before GPU limit (`--gpu-limit`).",
        )

    if ctx.obj.get("resource") is None:
        processor_type = ctx.obj.get("processor_type")
        if processor_type is None:
            raise click.UsageError(
                message="Processor type must be specified before GPU limit (`--gpu-limit`).",
            )

        if processor_type == PROCESSOR_TYPE_GPU:
            return click.prompt("GPUs (in vGPU)", type=click.FLOAT)


def image_url_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    processor_type = ctx.obj.get("processor_type")
    if processor_type is None:
        raise click.UsageError(
            message="Processor type must be specified before image URL (`--image-url`).",
        )

    choice = prompt_choices(
        "Image type",
        [
            "VESSL Managed Docker Image",
            "Public Docker Image",
            "Private Docker Image",
        ],
    )

    if choice == "Public Docker Image":
        ctx.obj["is_image_private"] = False
        return click.prompt("Image URL", type=click.STRING)
    elif choice == "Private Docker Image":
        ctx.obj["is_image_private"] = True
        return click.prompt("Image URL", type=click.STRING)
    else:
        ctx.obj["is_image_private"] = False
        images = list_kernel_images()
        images = [x for x in images if x.processor_type == processor_type]

        # Pretty print image name+URL choices
        name_maxlen = len(max(images, key=lambda x: len(x.name)).name)
        return prompt_choices(
            "Image",
            [
                (
                    f"{x.name}" + " " * (name_maxlen - len(x.name) + 2) + f"(URL: {x.image_url})",
                    x.image_url,
                )
                for x in images
            ],
        )


def upload_local_file_callback(
    ctx: click.Context, param: click.Parameter, value: List[str]
) -> List[str]:
    if value:
        for dir_spec in value:
            if ":" in dir_spec:
                local_path, _ = dir_spec.split(":")
            else:
                local_path = dir_spec

            if not local_path:
                raise click.BadOptionUsage(
                    option_name="--upload-local-file",
                    message=f"Invalid path {local_path}.",
                )

            if not os.path.exists(local_path):
                raise click.BadOptionUsage(
                    option_name="--upload-local-file",
                    message=f"{local_path} does not exist.",
                )
    return value


def docker_credentials_prompter(
    ctx: click.Context, param: click.Parameter, value: int
) -> Optional[int]:
    if ctx.obj.get("is_image_private"):
        creds = list_organization_credentials(
            types=["aws-access-key,docker-credentials"],
            components=["image,image"],
        )
        return prompt_choices(
            "Image",
            [
                (
                    f"{c.credentials_name} | {c.credentials_type} | (created by: '{c.created_by.username}')",
                    c.credentials_id,
                )
                for c in creds
            ],
        )
    return None


def worker_count_callback(ctx: click.Context, param: click.Parameter, value: int) -> int:
    if value is None:
        value = 1

    if value < 1:
        raise click.BadOptionUsage(
            option_name="--worker-count",
            message="num nodes (`--num-nodes`) must be a positive integer.",
        )

    ctx.obj["worker_count"] = value
    return value


def framework_type_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    worker_count = ctx.obj.get("worker_count")
    if worker_count == 1:
        return ""

    framework_type = prompt_choices("Processor Type", FRAMEWORK_TYPES)
    if framework_type == FRAMEWORK_TYPE_TENSORFLOW:
        raise click.BadOptionUsage(
            option_name="--framework-type",
            message="Only PyTorch distributed experiment is supported currently.",
        )
    return framework_type


@click.command(name="experiment", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "number",
    type=click.INT,
    required=True,
    prompter=experiment_number_prompter,
)
@organization_name_option
@project_name_option
def read(number: int):
    experiment = read_experiment(experiment_number=number)

    distributed_spec = "None"
    if experiment.is_distributed:
        if experiment.distributed_spec.framework_type == FRAMEWORK_TYPE_PYTORCH:
            distributed_spec = {
                "Framework Type": experiment.distributed_spec.framework_type,
                "PyTorch Spec": {
                    "Worker count": experiment.distributed_spec.pytorch_spec.worker_replicas,
                },
            }
        elif experiment.distributed_spec.framework_type == FRAMEWORK_TYPE_TENSORFLOW:
            distributed_spec = {
                "Framework Type": experiment.distributed_spec.framework_type,
                "TensorFlow Spec": experiment.distributed_spec.tensorflow_spec,
            }

    kernel_image = "None"
    if experiment.kernel_image:
        kernel_image = {
            "Name": experiment.kernel_image.name,
            "URL": experiment.kernel_image.image_url,
        }

    resource_spec = "None"
    if experiment.kernel_resource_spec:
        resource_spec = {
            "Name": experiment.kernel_resource_spec.name,
            "CPU Type": experiment.kernel_resource_spec.cpu_type,
            "CPU Limit": experiment.kernel_resource_spec.cpu_limit,
            "Memory Limit": experiment.kernel_resource_spec.memory_limit,
            "GPU Type": experiment.kernel_resource_spec.gpu_type,
            "GPU Limit": experiment.kernel_resource_spec.gpu_limit,
        }

    metrics_summary = "None"
    if experiment.metrics_summary.latest:
        metrics_keys = experiment.metrics_summary.latest.keys()
        metrics_summary = {}
        for key in metrics_keys:
            metrics_summary[key] = experiment.metrics_summary.latest[key].value

    print_data(
        {
            "ID": experiment.id,
            "Number": experiment.number,
            "Distributed": experiment.is_distributed,
            "Distributed spec": distributed_spec,
            "Local": experiment.is_local,
            "Status": experiment.status,
            "Created": truncate_datetime(experiment.created_dt),
            "Message": experiment.message,
            "Kernel image": kernel_image,
            "Resource spec": resource_spec,
            "Start command": experiment.start_command,
            "Metrics summary": metrics_summary,
        }
    )
    print_info(
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


@cli.vessl_command()
@organization_name_option
@project_name_option
@vessl_option(
    "--limit",
    type=click.INT,
    required=False,
    default=None,
)
def list(limit: Optional[int]):
    experiments = list_experiments(limit=limit)
    print_table(
        experiments,
        ["ID", "Number", "Distributed", "Status", "Created", "Message"],
        lambda x: [
            x.id,
            x.number,
            x.is_distributed,
            x.status,
            truncate_datetime(x.created_dt),
            x.message,
        ],
    )


command_option = vessl_option(
    "-x",
    "--command",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Start command"),
    help="Start command to execute in experiment container.",
)
processor_type_option = vessl_option(
    "--processor-type",
    type=click.Choice(("CPU", "GPU")),
    prompter=processor_type_prompter,
    callback=processor_type_callback,
    help="CPU or GPU (for custom resource only).",
)
cpu_limit_option = vessl_option(
    "--cpu-limit",
    type=click.FLOAT,
    prompter=cpu_limit_prompter,
    help="Number of vCPUs (for custom resource only).",
)
memory_limit_option = vessl_option(
    "--memory-limit",
    type=click.STRING,
    prompter=memory_limit_prompter,
    help="Memory limit (e.g. 4Gi) (for custom resource only).",
)
gpu_type_option = vessl_option(
    "--gpu-type",
    type=click.STRING,
    prompter=gpu_type_prompter,
    help="GPU type such as Tesla-K80 (for custom resource only).",
)
gpu_limit_option = vessl_option(
    "--gpu-limit",
    type=click.INT,
    prompter=gpu_limit_prompter,
    help="Number of GPU cores (for custom resource only).",
)
image_url_option = vessl_option(
    "-i",
    "--image-url",
    type=click.STRING,
    prompter=image_url_prompter,
    help="Kernel docker image URL",
)
docker_credentials_id_option = vessl_option(
    "--docker-credentials-id",
    type=click.INT,
    prompter=docker_credentials_prompter,
    help="Docker credential prompter",
)
message_option = click.option("-m", "--message", type=click.STRING)
termination_protection_option = click.option("--termination-protection", is_flag=True)
hyperparameter_option = click.option(
    "-h",
    "--hyperparameter",
    type=click.STRING,
    multiple=True,
    help="Hyperparameters. Format: [key]=[value], ex. `--hyperparameter lr=0.01`.",
)
secret_option = click.option(
    "--secret",
    type=click.STRING,
    multiple=True,
    help=(
        "Secret environment variables that are hidden in UI and logs. "
        "Format: [key]=[value], ex. `--secret DB_PASSWORD=vessl1234`."
    ),
)
dataset_option = click.option(
    "--dataset",
    type=click.STRING,
    multiple=True,
    help="Dataset mounts. Format: [mount_path]:[dataset_name]@[optional_dataset_version], ex. `--dataset /input:mnist@3bcd5f`.",
)
model_option = click.option(
    "--model",
    type=click.STRING,
    multiple=True,
    help="Model mounts. Format: [mount_path]:[model_repository_name]/[model_number], ex. `--model /input:pytorch-model/1` .",
)
git_ref_option = click.option(
    "--git-ref",
    type=click.STRING,
    multiple=True,
    help=f"Git repository mounts. Format: [mount_path]:github/[organization]/[repository]/[optional_commit], ex. `--git-ref {EXPERIMENT_WORKING_DIR}examples:github/vessl-ai/examples/3cd23dd`.",
)
git_diff_option = click.option(
    "--git-diff",
    type=click.STRING,
    help="Git diff file mounts. Format: [mount_path]:[volume_file_path]. This option is used only for reproducing existing experiments.",
)
upload_local_file_option = vessl_option(
    "--upload-local-file",
    type=click.STRING,
    multiple=True,
    callback=upload_local_file_callback,
    help="Upload local file. Format: [local_path] or [local_path]:[remote_path].",
)
use_vesslignore_option = vessl_option(
    "--use-vesslignore/--no-use-vesslignore",
    type=click.BOOL,
    default=True,
    help="\n".join(
        [
            "Choose whether to use .vesslignore files when uploading local files.",
            "A .vesslignore file contains patterns that applies relatively to the directory containing that .vesslignore file. Files that match any of those patterns are excluded from uploading.",
            "The syntax of .vesslignore is similar to that of .gitignore. Please consult the documentation for more details.",
        ]
    ),
)
upload_local_git_diff_option = vessl_option(
    "--upload-local-git-diff",
    is_flag=True,
    help="Upload local git commit hash and diff.",
)
archive_file_option = click.option(
    "--archive-file",
    type=click.STRING,
    help="Local archive file mounts. Format: [mount_path]:[archive_file_path]]. This option is used only for reproducing existing experiments.",
)
object_storage_option = vessl_option(
    "--object-storage",
    type=click.STRING,
    multiple=True,
    help="[ALPHA] Object storage mounts. Format: [upload|download]@[mount_path]:[object_storage_name]/[object_storage_path]. e.g. `--object-storage upload@/input:s3://some/bucket.path`.",
)
root_volume_size_option = click.option("--root-volume-size", type=click.STRING)
working_dir_option = click.option(
    "--working-dir", type=click.STRING, help=f"Defaults to `{EXPERIMENT_WORKING_DIR}`."
)
output_dir_option = click.option(
    "--output-dir",
    type=click.STRING,
    default=MOUNT_PATH_OUTPUT,
    help="Directory to store experiment output files. Defaults to `/output/`.",
)
worker_count_option = vessl_option(
    "--worker-count",
    type=click.INT,
    callback=worker_count_callback,
    help="The number of nodes to run an experiment. Defaults to 1.",
)
framework_type_option = vessl_option(
    "--framework-type",
    type=click.STRING,
    prompter=framework_type_prompter,
    help="Framework type option. Defaults to `pytorch`.",
)
service_account_name_option = vessl_option(
    "--service-account",
    type=click.STRING,
    default="",
    help="Kubernetes service account name.",
)


@cli.vessl_command()
@click.pass_context
@organization_name_option
@project_name_option
@cluster_option
@vessl_option(
    "--node",
    type=click.STRING,
    multiple=True,
    help="Cluster nodes. Defaults to all nodes in cluster.",
)
@resource_option
@processor_type_option
@cpu_limit_option
@memory_limit_option
@gpu_type_option
@gpu_limit_option
@image_url_option
@docker_credentials_id_option
@message_option
@termination_protection_option
@hyperparameter_option
@secret_option
@dataset_option
@model_option
@git_ref_option
@git_diff_option
@upload_local_file_option
@use_vesslignore_option
@upload_local_git_diff_option
@archive_file_option
@object_storage_option
@root_volume_size_option
@working_dir_option
@output_dir_option
@command_option
@worker_count_option
@framework_type_option
@service_account_name_option
def create(
    ctx,
    cluster: str,
    node: List[str],
    command: str,
    resource: str,
    processor_type: str,
    cpu_limit: float,
    memory_limit: str,
    gpu_type: str,
    gpu_limit: int,
    image_url: str,
    docker_credentials_id: Optional[int],
    message: str,
    termination_protection: bool,
    hyperparameter: List[str],
    secret: List[str],
    dataset: List[str],
    model: List[str],
    git_ref: List[str],
    git_diff: str,
    upload_local_file: List[str],
    use_vesslignore: bool,
    upload_local_git_diff: bool,
    archive_file: str,
    object_storage: List[str],
    root_volume_size: str,
    working_dir: str,
    output_dir: str,
    worker_count: int,
    framework_type: str,
    service_account: str,
):
    experiment = create_experiment(
        cluster_name=cluster,
        cluster_node_names=node,
        start_command=command,
        kernel_resource_spec_name=resource,
        processor_type=processor_type,
        cpu_limit=cpu_limit,
        memory_limit=memory_limit,
        gpu_type=gpu_type,
        gpu_limit=gpu_limit,
        kernel_image_url=image_url,
        docker_credentials_id=docker_credentials_id,
        message=message,
        termination_protection=termination_protection,
        hyperparameters=hyperparameter,
        secrets=secret,
        dataset_mounts=dataset,
        model_mounts=model,
        git_ref_mounts=git_ref,
        git_diff_mount=git_diff,
        local_files=upload_local_file,
        use_vesslignore=use_vesslignore,
        upload_local_git_diff=upload_local_git_diff,
        archive_file_mount=archive_file,
        object_storage_mounts=object_storage,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
        worker_count=worker_count,
        framework_type=framework_type,
        service_account=service_account,
    )
    ctx.obj["experiment_number"] = experiment.number
    print_success(
        f"Created '{experiment.number}'.\n"
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


worker_number_option = vessl_option(
    "--worker-number",
    type=click.INT,
    default=0,
    help="Worker number (for distributed experiment only).",
)


@cli.vessl_command()
@vessl_argument(
    "number",
    type=click.INT,
    required=True,
    prompter=experiment_number_prompter,
)
@click.option(
    "--tail",
    type=click.INT,
    default=200,
    help="Number of lines to display (from the end).",
)
@click.option(
    "-f",
    "--follow",
    is_flag=True,
    default=False,
)
@organization_name_option
@project_name_option
@worker_number_option
def logs(
    number: int,
    tail: int,
    follow: bool,
    worker_number: int,
):
    if not follow:
        logs = list_experiment_logs(experiment_number=number, tail=tail, worker_numer=worker_number)
        print_logs(logs)
        print_info(f"Displayed last {len(logs)} lines of #{number}.")
        return

    after = 0

    experiment_finished_dt = None
    while True:
        if (
            read_experiment(number).status not in ["pending", "running"]
            and experiment_finished_dt is None
        ):
            experiment_finished_dt = time.time()

        if experiment_finished_dt is not None and (time.time() - experiment_finished_dt) > 5:
            break

        logs = list_experiment_logs(
            experiment_number=number,
            before=int(time.time() - 5),
            after=after,
            worker_numer=worker_number,
        )
        print_logs(logs)
        if len(logs) > 0:
            after = logs[-1].timestamp + 0.000001

        time.sleep(3)


@cli.vessl_command()
@vessl_argument(
    "number",
    type=click.INT,
    required=True,
    prompter=experiment_number_prompter,
)
@organization_name_option
@project_name_option
@worker_number_option
def list_output(
    number: int,
    worker_number: int,
):
    files = list_experiment_output_files(
        experiment_number=number,
        need_download_url=False,
        recursive=True,
        worker_number=worker_number,
    )
    print_volume_files(files)


@cli.vessl_command()
@vessl_argument(
    "number",
    type=click.INT,
    required=True,
    prompter=experiment_number_prompter,
)
@click.option(
    "-p",
    "--path",
    type=click.Path(),
    default=os.path.join(os.getcwd(), "output"),
    help="Path to store downloads. Defaults to `./output`.",
)
@worker_number_option
@organization_name_option
@project_name_option
def download_output(
    number: int,
    path: str,
    worker_number: int,
):
    download_experiment_output_files(
        experiment_number=number,
        dest_path=path,
        worker_number=worker_number,
    )
    print_success(f"Downloaded experiment output to {path}.")


@cli.vessl_command()
@vessl_argument(
    "number",
    type=click.INT,
    required=True,
    prompter=experiment_number_prompter,
)
@organization_name_option
@project_name_option
def terminate(number: int):
    experiment = terminate_experiment(experiment_number=number)
    print_success(
        f"Terminated '#{experiment.number}'.\n"
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


@cli.vessl_command()
@vessl_argument(
    "number",
    type=click.INT,
    required=True,
    prompter=experiment_number_prompter,
)
@organization_name_option
@project_name_option
def delete(number: int):
    delete_experiment(experiment_number=number)
    print_success(f"Deleted '#{number}'.\n")



import base64
import os
import time
from typing import Optional

from vessl.openapi_client import (
    ProtoTag,
    ResponseVolumeV2FederateInfo,
    ResponseVolumeV2Info,
    StorageGoogleStorageFederationToken,
    StorageS3FederationToken,
    VolumeV2CreateAPIInput,
    VolumeV2FederateAPIInput,
)
from vessl import vessl_api
from vessl.organization import _get_organization_name
from vessl.storage.abstract_volume import AbstractVolume
from vessl.storage.file import VolumeFile
from vessl.storage.gcs import GCSVolume
from vessl.storage.s3 import S3Volume
from vessl.util import constant
from vessl.util.common import safe_cast
from vessl.util.fmt import format_size
from vessl.volume import VolumePathRef, parse_volume_url


def list_volumes(
    storage_name: str, keyword: Optional[str] = None, **kwargs
) -> list[ResponseVolumeV2Info]:
    """List volumes in provided storage name with default organization.
    If you want to override the default organization, then pass `organization_name` as `**kwargs`.

    Args:
        storage_name: name of the storage
        keyword: optional search keyword

    Example:
        ```python
        vessl.storage.list_volumes(storage_name="my-storage")
        ```
    """
    query_keys = {"limit", "offset"}
    query_kwargs = {k: v for k, v in kwargs.items() if k in query_keys}

    return vessl_api.volume_v2_list_api(
        organization_name=_get_organization_name(**kwargs),
        storage_name=storage_name,
        keyword=keyword,
        **query_kwargs,
    ).results


def create_volume(name: str, storage_name: str, tags: tuple[str, ...] = (), **kwargs):
    """Create volume in provided storage name with default organization.
    If you want to override the default organization, then pass `organization_name` as `**kwargs`.

    Args:
        name: The name of the volume.
        storage_name: The name of the storage.
        tags: The tags of the volume.

    Example:
        ```python
        vessl.storage.create_volume(
            name="my-volume",
            storage_name="my-storage",
            tags=("my-tag1", "my-tag2"),
        )
        ```
    """
    vessl_api.volume_v2_create_api(
        organization_name=_get_organization_name(**kwargs),
        storage_name=storage_name,
        volume_v2_create_api_input=VolumeV2CreateAPIInput(
            volume_name=name,
            tags=[ProtoTag(name=tag) for tag in tags],
        ),
    )


def delete_volume(name: str, storage_name: str, **kwargs):
    """Delete volume in provided storage name with default organization.
    If you want to override the default organization, then pass `organization_name` as `**kwargs`.

    Args:
        name: The name of the volume.
        storage_name: The name of the storage.

    Example:
        ```python
        vessl.storage.delete_volume(name="my-volume", storage_name="my-storage")
        ```
    """
    vessl_api.volume_v2_delete_api(
        organization_name=_get_organization_name(**kwargs),
        storage_name=storage_name,
        volume_name=name,
    )


def volume_federate(
    storage_name: str, volume_name: str, federation_type: str, **kwargs
) -> ResponseVolumeV2FederateInfo:
    """
    Get federation information for a Volume.

    Args:
        storage_name (str): Name of the storage.
        volume_name (str): Name of the volume in the storage.

    Example:
        ```python
        vessl.storage.volume_federate(
            storage_name="my-storage-1",
            volume_name="my-volume-1",
        )
        ```
    """

    return vessl_api.volume_v2_federate_api(
        organization_name=_get_organization_name(**kwargs),
        storage_name=storage_name,
        volume_name=volume_name,
        volume_v2_federate_api_input=VolumeV2FederateAPIInput(federation_type=federation_type),
    )


def _get_volume_with_federate(
    source_storage_name: str, source_volume_name: str, federation_type: str
) -> AbstractVolume:
    fed = volume_federate(
        storage_name=source_storage_name,
        volume_name=source_volume_name,
        federation_type=federation_type,
    )

    insecure_skip_tls_verify = safe_cast(os.environ.get('VESSL_INSECURE_SKIP_TLS_VERIFY'), bool, False)
    verify_tls = not insecure_skip_tls_verify

    if fed.type == "s3":
        s3_token: Optional[StorageS3FederationToken] = fed.s3_token
        if s3_token is None:
            return S3Volume(
                bucket_name=fed.bucket_name,
                base_path=fed.path,
                verify_tls=verify_tls,
            )
        else:
            return S3Volume(
                aws_access_key_id=s3_token.access_key_id,
                aws_secret_access_key=s3_token.secret_access_key,
                aws_session_token=s3_token.session_token,
                bucket_name=fed.bucket_name,
                base_path=fed.path,
                region=fed.region,
                endpoint_url=fed.endpoint,
                force_path_style=fed.force_path_style,
                verify_tls=verify_tls,
            )
    elif fed.type == "gcs":
        gcs_token: StorageGoogleStorageFederationToken = fed.gcs_token
        cred_decoded = base64.b64decode(gcs_token.base64_credentials).decode("utf-8")
        return GCSVolume(
            bucket_name=fed.bucket_name,
            base_path=fed.path,
            gcp_api_key_json=cred_decoded,
        )
    else:
        raise NotImplementedError(f"invalid type: {fed.type}")


def copy_volume_file(source: VolumePathRef, dest: VolumePathRef):
    t2t = (source.type, dest.type)
    if t2t == (constant.VOLUME_TYPE_VOLUME, constant.VOLUME_TYPE_LOCAL):
        download_volume_file(
            source_storage_name=source.storage_name,
            source_volume_name=source.volume_name,
            dest_path=dest.local_path,
        )
    elif t2t == (constant.VOLUME_TYPE_LOCAL, constant.VOLUME_TYPE_VOLUME):
        upload_volume_file(
            source_path=source.local_path,
            dest_storage_name=dest.storage_name,
            dest_volume_name=dest.volume_name,
            dest_path=dest.volume_path,
        )
    else:
        raise NotImplementedError(
            "The sv:// scheme for volume operations is deprecated. "
            "For backward compatibility, please use vessl.copy_volume_file() instead of vessl.storage.copy_volume_file()."
        )


def list_volume_files(storage_name: str, volume_name: str, path: str = "") -> list[VolumeFile]:
    """
    List all files in a volume.

    Arguments:
         storage_name (str): Name of the storage.
         volume_name (str): Name of the volume in the storage.
         path (str, optional): Path of directory to list. Defaults to "".

    Example:
        ```python
        vessl.storage.list_volume_files(
            storage_name="my-storage",
            volume_name="my-volume",
        )
        ```
    """
    volume = _get_volume_with_federate(storage_name, volume_name, federation_type="read")
    return volume.list(path)


def download_volume_file(source_storage_name: str, source_volume_name: str, dest_path: str):
    """
    Download a file from a volume.

    Arguments:
         source_storage_name (str): Name of the storage.
         source_volume_name (str): Name of the volume in the storage.
         dest_path (str): Path of local directory to download the file to.

    Examples:
        ```python
        vessl.storage.download_volume_file(
            source_storage_name="my-storage",
            source_volume_name="my-volume",
            dest_path="models"
        )
        ```
    """
    os.makedirs(dest_path, exist_ok=True)

    volume = _get_volume_with_federate(
        source_storage_name, source_volume_name, federation_type="read"
    )

    progress_last_printed = time.time()

    def progress_callback(
        total_file_count,
        current_file_index,
        current_file_size,
        current_file_downloaded,
        total_downloaded,
    ):
        nonlocal progress_last_printed

        now = time.time()
        if now < progress_last_printed + 5.0:
            return
        progress_last_printed = now

        print(
            f"Downloading: {current_file_index:4}/{total_file_count:4} file(s): "
            + f"{format_size(current_file_downloaded)}/{format_size(current_file_size)} "
            + f"({current_file_downloaded/current_file_size*100:5.2f}%), "
            + f"total {format_size(total_downloaded)}"
        )

    volume.download_directory("", destination=dest_path, progress_callback=progress_callback)


def upload_volume_file(
    source_path: str, dest_storage_name: str, dest_volume_name: str, dest_path: str = ""
):
    """
    Upload a file from a volume.

    Arguments:
         source_path (str): Path of local directory or file to upload.
         dest_storage_name (str): Name of the storage.
         dest_volume_name (str): Name of the volume in the storage.
         dest_path (str): Path of volume directory to upload the file to. Defaults to "".

    Examples:
        ```python
        vessl.storage.upload_volume_file(
            source_path="/path/to/file",
            dest_storage_name="my-storage",
            dest_volume_name="my-volume",
        )
        ```
    """
    volume = _get_volume_with_federate(dest_storage_name, dest_volume_name, federation_type="write")

    progress_last_printed = time.time()

    def progress_callback(
        total_file_count,
        current_file_index,
        current_file_size,
        current_file_downloaded,
        total_downloaded,
    ):
        nonlocal progress_last_printed

        now = time.time()
        if now < progress_last_printed + 5.0:
            return
        progress_last_printed = now

        print(
            f"Downloading: {current_file_index:4}/{total_file_count:4} file(s): "
            + f"{format_size(current_file_downloaded)}/{format_size(current_file_size)} "
            + f"({current_file_downloaded/current_file_size*100:5.2f}%), "
            + f"total {format_size(total_downloaded)}"
        )

    if os.path.isdir(source_path):
        volume.upload_directory(
            source_path, destination=dest_path, progress_callback=progress_callback
        )
    else:
        destination = os.path.join(dest_path, os.path.basename(source_path))
        volume.upload_file(source_path, destination=destination)


def delete_volume_file(storage_name: str, volume_name: str, path: str, recursive: bool = False):
    """
    Delete a file from a volume.

    Arguments:
         storage_name (str): Name of the storage.
         volume_name (str): Name of the volume in the storage.
         path (str): Path of directory or file in volume to delete.
         recursive (bool, optional): If true, delete all files in this directory. Defaults to False.

    Examples:
        ```python
        vessl.storage.delete_volume_file(
            storage_name="my-storage",
            volume_name="my-volume",
            path="model.pth"
        )
        ```
    """
    volume = _get_volume_with_federate(storage_name, volume_name, federation_type="write")

    # Even if recursive=True, always call delete_file() to ensure the file is deleted
    # when the input is a file instead of a directory.
    if recursive:
        volume.delete_directory(path)
    volume.delete_file(path)



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ProjectNotificationSettingsUpdateAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'enabled': 'bool',
        'message_template': 'str',
        'on_run_complete': 'bool',
        'on_run_failure': 'bool',
        'on_run_start': 'bool'
    }

    attribute_map = {
        'enabled': 'enabled',
        'message_template': 'message_template',
        'on_run_complete': 'on_run_complete',
        'on_run_failure': 'on_run_failure',
        'on_run_start': 'on_run_start'
    }

    def __init__(self, enabled=None, message_template=None, on_run_complete=None, on_run_failure=None, on_run_start=None, local_vars_configuration=None):  # noqa: E501
        """ProjectNotificationSettingsUpdateAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._enabled = None
        self._message_template = None
        self._on_run_complete = None
        self._on_run_failure = None
        self._on_run_start = None
        self.discriminator = None

        if enabled is not None:
            self.enabled = enabled
        if message_template is not None:
            self.message_template = message_template
        if on_run_complete is not None:
            self.on_run_complete = on_run_complete
        if on_run_failure is not None:
            self.on_run_failure = on_run_failure
        if on_run_start is not None:
            self.on_run_start = on_run_start

    @property
    def enabled(self):
        """Gets the enabled of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The enabled of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this ProjectNotificationSettingsUpdateAPIInput.


        :param enabled: The enabled of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type enabled: bool
        """

        self._enabled = enabled

    @property
    def message_template(self):
        """Gets the message_template of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The message_template of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._message_template

    @message_template.setter
    def message_template(self, message_template):
        """Sets the message_template of this ProjectNotificationSettingsUpdateAPIInput.


        :param message_template: The message_template of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type message_template: str
        """

        self._message_template = message_template

    @property
    def on_run_complete(self):
        """Gets the on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._on_run_complete

    @on_run_complete.setter
    def on_run_complete(self, on_run_complete):
        """Sets the on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.


        :param on_run_complete: The on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type on_run_complete: bool
        """

        self._on_run_complete = on_run_complete

    @property
    def on_run_failure(self):
        """Gets the on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._on_run_failure

    @on_run_failure.setter
    def on_run_failure(self, on_run_failure):
        """Sets the on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.


        :param on_run_failure: The on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type on_run_failure: bool
        """

        self._on_run_failure = on_run_failure

    @property
    def on_run_start(self):
        """Gets the on_run_start of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The on_run_start of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._on_run_start

    @on_run_start.setter
    def on_run_start(self, on_run_start):
        """Sets the on_run_start of this ProjectNotificationSettingsUpdateAPIInput.


        :param on_run_start: The on_run_start of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type on_run_start: bool
        """

        self._on_run_start = on_run_start

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectNotificationSettingsUpdateAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProjectNotificationSettingsUpdateAPIInput):
            return True

        return self.to_dict() != other.to_dict()



import webbrowser

bypass_browsers = ["www-browser", "links", "lynx", "elinks", "w3m"]


def try_open_gui_browser(url: str) -> bool:
    """
    Open a browser window to the specified URL.
    """

    try:
        default_browser = webbrowser.get()
    except webbrowser.Error:
        return False

    if default_browser in bypass_browsers:
        return False  # Don't open a browser if the default is a text-based browser

    webbrowser.open(url)

    return True



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmArtifactEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'child_artifacts': 'list[OrmArtifact]',
        'created_by': 'OrmUser',
        'llm_documents': 'list[OrmLLMDocument]',
        'locator': 'list[OrmVolume]',
        'models': 'list[OrmModel]',
        'organization': 'OrmOrganization',
        'origin_volume': 'OrmVolume',
        'parent_artifact': 'OrmArtifact',
        'run_exec': 'OrmRunExecution',
        'workload': 'OrmWorkload'
    }

    attribute_map = {
        'child_artifacts': 'child_artifacts',
        'created_by': 'created_by',
        'llm_documents': 'llm_documents',
        'locator': 'locator',
        'models': 'models',
        'organization': 'organization',
        'origin_volume': 'origin_volume',
        'parent_artifact': 'parent_artifact',
        'run_exec': 'run_exec',
        'workload': 'workload'
    }

    def __init__(self, child_artifacts=None, created_by=None, llm_documents=None, locator=None, models=None, organization=None, origin_volume=None, parent_artifact=None, run_exec=None, workload=None, local_vars_configuration=None):  # noqa: E501
        """OrmArtifactEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._child_artifacts = None
        self._created_by = None
        self._llm_documents = None
        self._locator = None
        self._models = None
        self._organization = None
        self._origin_volume = None
        self._parent_artifact = None
        self._run_exec = None
        self._workload = None
        self.discriminator = None

        if child_artifacts is not None:
            self.child_artifacts = child_artifacts
        if created_by is not None:
            self.created_by = created_by
        if llm_documents is not None:
            self.llm_documents = llm_documents
        if locator is not None:
            self.locator = locator
        if models is not None:
            self.models = models
        if organization is not None:
            self.organization = organization
        if origin_volume is not None:
            self.origin_volume = origin_volume
        if parent_artifact is not None:
            self.parent_artifact = parent_artifact
        if run_exec is not None:
            self.run_exec = run_exec
        if workload is not None:
            self.workload = workload

    @property
    def child_artifacts(self):
        """Gets the child_artifacts of this OrmArtifactEdges.  # noqa: E501


        :return: The child_artifacts of this OrmArtifactEdges.  # noqa: E501
        :rtype: list[OrmArtifact]
        """
        return self._child_artifacts

    @child_artifacts.setter
    def child_artifacts(self, child_artifacts):
        """Sets the child_artifacts of this OrmArtifactEdges.


        :param child_artifacts: The child_artifacts of this OrmArtifactEdges.  # noqa: E501
        :type child_artifacts: list[OrmArtifact]
        """

        self._child_artifacts = child_artifacts

    @property
    def created_by(self):
        """Gets the created_by of this OrmArtifactEdges.  # noqa: E501


        :return: The created_by of this OrmArtifactEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this OrmArtifactEdges.


        :param created_by: The created_by of this OrmArtifactEdges.  # noqa: E501
        :type created_by: OrmUser
        """

        self._created_by = created_by

    @property
    def llm_documents(self):
        """Gets the llm_documents of this OrmArtifactEdges.  # noqa: E501


        :return: The llm_documents of this OrmArtifactEdges.  # noqa: E501
        :rtype: list[OrmLLMDocument]
        """
        return self._llm_documents

    @llm_documents.setter
    def llm_documents(self, llm_documents):
        """Sets the llm_documents of this OrmArtifactEdges.


        :param llm_documents: The llm_documents of this OrmArtifactEdges.  # noqa: E501
        :type llm_documents: list[OrmLLMDocument]
        """

        self._llm_documents = llm_documents

    @property
    def locator(self):
        """Gets the locator of this OrmArtifactEdges.  # noqa: E501


        :return: The locator of this OrmArtifactEdges.  # noqa: E501
        :rtype: list[OrmVolume]
        """
        return self._locator

    @locator.setter
    def locator(self, locator):
        """Sets the locator of this OrmArtifactEdges.


        :param locator: The locator of this OrmArtifactEdges.  # noqa: E501
        :type locator: list[OrmVolume]
        """

        self._locator = locator

    @property
    def models(self):
        """Gets the models of this OrmArtifactEdges.  # noqa: E501


        :return: The models of this OrmArtifactEdges.  # noqa: E501
        :rtype: list[OrmModel]
        """
        return self._models

    @models.setter
    def models(self, models):
        """Sets the models of this OrmArtifactEdges.


        :param models: The models of this OrmArtifactEdges.  # noqa: E501
        :type models: list[OrmModel]
        """

        self._models = models

    @property
    def organization(self):
        """Gets the organization of this OrmArtifactEdges.  # noqa: E501


        :return: The organization of this OrmArtifactEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmArtifactEdges.


        :param organization: The organization of this OrmArtifactEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def origin_volume(self):
        """Gets the origin_volume of this OrmArtifactEdges.  # noqa: E501


        :return: The origin_volume of this OrmArtifactEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._origin_volume

    @origin_volume.setter
    def origin_volume(self, origin_volume):
        """Sets the origin_volume of this OrmArtifactEdges.


        :param origin_volume: The origin_volume of this OrmArtifactEdges.  # noqa: E501
        :type origin_volume: OrmVolume
        """

        self._origin_volume = origin_volume

    @property
    def parent_artifact(self):
        """Gets the parent_artifact of this OrmArtifactEdges.  # noqa: E501


        :return: The parent_artifact of this OrmArtifactEdges.  # noqa: E501
        :rtype: OrmArtifact
        """
        return self._parent_artifact

    @parent_artifact.setter
    def parent_artifact(self, parent_artifact):
        """Sets the parent_artifact of this OrmArtifactEdges.


        :param parent_artifact: The parent_artifact of this OrmArtifactEdges.  # noqa: E501
        :type parent_artifact: OrmArtifact
        """

        self._parent_artifact = parent_artifact

    @property
    def run_exec(self):
        """Gets the run_exec of this OrmArtifactEdges.  # noqa: E501


        :return: The run_exec of this OrmArtifactEdges.  # noqa: E501
        :rtype: OrmRunExecution
        """
        return self._run_exec

    @run_exec.setter
    def run_exec(self, run_exec):
        """Sets the run_exec of this OrmArtifactEdges.


        :param run_exec: The run_exec of this OrmArtifactEdges.  # noqa: E501
        :type run_exec: OrmRunExecution
        """

        self._run_exec = run_exec

    @property
    def workload(self):
        """Gets the workload of this OrmArtifactEdges.  # noqa: E501


        :return: The workload of this OrmArtifactEdges.  # noqa: E501
        :rtype: OrmWorkload
        """
        return self._workload

    @workload.setter
    def workload(self, workload):
        """Sets the workload of this OrmArtifactEdges.


        :param workload: The workload of this OrmArtifactEdges.  # noqa: E501
        :type workload: OrmWorkload
        """

        self._workload = workload

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmArtifactEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmArtifactEdges):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ResponseOrganizationCredentialProviderInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'base_url': 'str',
        'created_dt': 'datetime',
        'id': 'int',
        'identity': 'str',
        'is_vessl_managed': 'bool',
        'kind': 'str',
        'name': 'str'
    }

    attribute_map = {
        'base_url': 'base_url',
        'created_dt': 'created_dt',
        'id': 'id',
        'identity': 'identity',
        'is_vessl_managed': 'is_vessl_managed',
        'kind': 'kind',
        'name': 'name'
    }

    def __init__(self, base_url=None, created_dt=None, id=None, identity=None, is_vessl_managed=None, kind=None, name=None, local_vars_configuration=None):  # noqa: E501
        """ResponseOrganizationCredentialProviderInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._base_url = None
        self._created_dt = None
        self._id = None
        self._identity = None
        self._is_vessl_managed = None
        self._kind = None
        self._name = None
        self.discriminator = None

        self.base_url = base_url
        self.created_dt = created_dt
        self.id = id
        self.identity = identity
        if is_vessl_managed is not None:
            self.is_vessl_managed = is_vessl_managed
        self.kind = kind
        self.name = name

    @property
    def base_url(self):
        """Gets the base_url of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The base_url of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: str
        """
        return self._base_url

    @base_url.setter
    def base_url(self, base_url):
        """Sets the base_url of this ResponseOrganizationCredentialProviderInfo.


        :param base_url: The base_url of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type base_url: str
        """
        if self.local_vars_configuration.client_side_validation and base_url is None:  # noqa: E501
            raise ValueError("Invalid value for `base_url`, must not be `None`")  # noqa: E501

        self._base_url = base_url

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The created_dt of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseOrganizationCredentialProviderInfo.


        :param created_dt: The created_dt of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def id(self):
        """Gets the id of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The id of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseOrganizationCredentialProviderInfo.


        :param id: The id of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def identity(self):
        """Gets the identity of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The identity of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: str
        """
        return self._identity

    @identity.setter
    def identity(self, identity):
        """Sets the identity of this ResponseOrganizationCredentialProviderInfo.


        :param identity: The identity of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type identity: str
        """
        if self.local_vars_configuration.client_side_validation and identity is None:  # noqa: E501
            raise ValueError("Invalid value for `identity`, must not be `None`")  # noqa: E501

        self._identity = identity

    @property
    def is_vessl_managed(self):
        """Gets the is_vessl_managed of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The is_vessl_managed of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: bool
        """
        return self._is_vessl_managed

    @is_vessl_managed.setter
    def is_vessl_managed(self, is_vessl_managed):
        """Sets the is_vessl_managed of this ResponseOrganizationCredentialProviderInfo.


        :param is_vessl_managed: The is_vessl_managed of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type is_vessl_managed: bool
        """

        self._is_vessl_managed = is_vessl_managed

    @property
    def kind(self):
        """Gets the kind of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The kind of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: str
        """
        return self._kind

    @kind.setter
    def kind(self, kind):
        """Sets the kind of this ResponseOrganizationCredentialProviderInfo.


        :param kind: The kind of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type kind: str
        """
        if self.local_vars_configuration.client_side_validation and kind is None:  # noqa: E501
            raise ValueError("Invalid value for `kind`, must not be `None`")  # noqa: E501

        self._kind = kind

    @property
    def name(self):
        """Gets the name of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501


        :return: The name of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseOrganizationCredentialProviderInfo.


        :param name: The name of this ResponseOrganizationCredentialProviderInfo.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseOrganizationCredentialProviderInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseOrganizationCredentialProviderInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmModelEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'artifact': 'OrmArtifact',
        'artifact_volume': 'OrmVolume',
        'created_by': 'OrmUser',
        'experiment': 'OrmExperiment',
        'model_generated_experiments': 'list[OrmModelGeneratedExperiment]',
        'model_repository': 'OrmModelRepository',
        'tags': 'list[OrmTag]'
    }

    attribute_map = {
        'artifact': 'artifact',
        'artifact_volume': 'artifact_volume',
        'created_by': 'created_by',
        'experiment': 'experiment',
        'model_generated_experiments': 'model_generated_experiments',
        'model_repository': 'model_repository',
        'tags': 'tags'
    }

    def __init__(self, artifact=None, artifact_volume=None, created_by=None, experiment=None, model_generated_experiments=None, model_repository=None, tags=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._artifact = None
        self._artifact_volume = None
        self._created_by = None
        self._experiment = None
        self._model_generated_experiments = None
        self._model_repository = None
        self._tags = None
        self.discriminator = None

        if artifact is not None:
            self.artifact = artifact
        if artifact_volume is not None:
            self.artifact_volume = artifact_volume
        if created_by is not None:
            self.created_by = created_by
        if experiment is not None:
            self.experiment = experiment
        if model_generated_experiments is not None:
            self.model_generated_experiments = model_generated_experiments
        if model_repository is not None:
            self.model_repository = model_repository
        if tags is not None:
            self.tags = tags

    @property
    def artifact(self):
        """Gets the artifact of this OrmModelEdges.  # noqa: E501


        :return: The artifact of this OrmModelEdges.  # noqa: E501
        :rtype: OrmArtifact
        """
        return self._artifact

    @artifact.setter
    def artifact(self, artifact):
        """Sets the artifact of this OrmModelEdges.


        :param artifact: The artifact of this OrmModelEdges.  # noqa: E501
        :type artifact: OrmArtifact
        """

        self._artifact = artifact

    @property
    def artifact_volume(self):
        """Gets the artifact_volume of this OrmModelEdges.  # noqa: E501


        :return: The artifact_volume of this OrmModelEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._artifact_volume

    @artifact_volume.setter
    def artifact_volume(self, artifact_volume):
        """Sets the artifact_volume of this OrmModelEdges.


        :param artifact_volume: The artifact_volume of this OrmModelEdges.  # noqa: E501
        :type artifact_volume: OrmVolume
        """

        self._artifact_volume = artifact_volume

    @property
    def created_by(self):
        """Gets the created_by of this OrmModelEdges.  # noqa: E501


        :return: The created_by of this OrmModelEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this OrmModelEdges.


        :param created_by: The created_by of this OrmModelEdges.  # noqa: E501
        :type created_by: OrmUser
        """

        self._created_by = created_by

    @property
    def experiment(self):
        """Gets the experiment of this OrmModelEdges.  # noqa: E501


        :return: The experiment of this OrmModelEdges.  # noqa: E501
        :rtype: OrmExperiment
        """
        return self._experiment

    @experiment.setter
    def experiment(self, experiment):
        """Sets the experiment of this OrmModelEdges.


        :param experiment: The experiment of this OrmModelEdges.  # noqa: E501
        :type experiment: OrmExperiment
        """

        self._experiment = experiment

    @property
    def model_generated_experiments(self):
        """Gets the model_generated_experiments of this OrmModelEdges.  # noqa: E501


        :return: The model_generated_experiments of this OrmModelEdges.  # noqa: E501
        :rtype: list[OrmModelGeneratedExperiment]
        """
        return self._model_generated_experiments

    @model_generated_experiments.setter
    def model_generated_experiments(self, model_generated_experiments):
        """Sets the model_generated_experiments of this OrmModelEdges.


        :param model_generated_experiments: The model_generated_experiments of this OrmModelEdges.  # noqa: E501
        :type model_generated_experiments: list[OrmModelGeneratedExperiment]
        """

        self._model_generated_experiments = model_generated_experiments

    @property
    def model_repository(self):
        """Gets the model_repository of this OrmModelEdges.  # noqa: E501


        :return: The model_repository of this OrmModelEdges.  # noqa: E501
        :rtype: OrmModelRepository
        """
        return self._model_repository

    @model_repository.setter
    def model_repository(self, model_repository):
        """Sets the model_repository of this OrmModelEdges.


        :param model_repository: The model_repository of this OrmModelEdges.  # noqa: E501
        :type model_repository: OrmModelRepository
        """

        self._model_repository = model_repository

    @property
    def tags(self):
        """Gets the tags of this OrmModelEdges.  # noqa: E501


        :return: The tags of this OrmModelEdges.  # noqa: E501
        :rtype: list[OrmTag]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this OrmModelEdges.


        :param tags: The tags of this OrmModelEdges.  # noqa: E501
        :type tags: list[OrmTag]
        """

        self._tags = tags

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelEdges):
            return True

        return self.to_dict() != other.to_dict()



import os
import tempfile
from pathlib import Path

import humanfriendly
from dotenv import dotenv_values

envvar = {**os.environ, **dotenv_values(".env")}

VESSL_ENV = envvar.get(
    "VESSL_ENV", "prod"
)  # default prod because cli and sdk should be packaged without .env file


# should be called at the end of this file as other values could be set across thie file, see below
# you should add any new envvar related constants here
def _print_envvar_related_constants_on_dev():
    if VESSL_ENV != "prod":
        print("Running in dev mode.. showing envvar related constants")
        print(
            f"""
>> VESSL_ENV: {VESSL_ENV}
>> WEB_HOST: {WEB_HOST}
>> API_HOST: {API_HOST}
>> SENTRY_DSN: {SENTRY_DSN}
>> DEFAULT_VESSL_SIDECAR_HOST: {DEFAULT_VESSL_SIDECAR_HOST}
>> PARALLEL_WORKERS: {PARALLEL_WORKERS}
"""
        )


VESSL_LOG_LEVEL_DEBUG = "DEBUG"
VESSL_LOG_LEVEL_INFO = "INFO"
VESSL_LOG_LEVEL_WARNING = "WARNING"
VESSL_LOG_LEVEL_ERROR = "ERROR"
VESSL_LOG_LEVEL_LEVELS = [
    VESSL_LOG_LEVEL_DEBUG,
    VESSL_LOG_LEVEL_INFO,
    VESSL_LOG_LEVEL_WARNING,
    VESSL_LOG_LEVEL_ERROR,
]

VESSL_LOG_LEVEL = (
    envvar.get("VESSL_LOG")
    if envvar.get("VESSL_LOG") in VESSL_LOG_LEVEL_LEVELS
    else VESSL_LOG_LEVEL_WARNING
)
VESSL_LOG_FAIL_ON_ERROR = envvar.get("VESSL_LOG_FAIL_ON_ERROR", "false").lower() == "true"

DEFAULT_WEB_HOST = "https://app.vessl.ai" if VESSL_ENV == "prod" else "https://app.dev.vssl.ai"
WEB_HOST = envvar.get("VESSL_WEB_HOST", DEFAULT_WEB_HOST)
DEFAULT_API_HOST = "https://api.vessl.ai" if VESSL_ENV == "prod" else "https://api.dev.vssl.ai"
API_HOST = envvar.get("VESSL_API_HOST", DEFAULT_API_HOST)

SENTRY_DSN = envvar.get(
    "SENTRY_DSN",
    "https://e46fcd750b3a443fbd5b9dbc970e4ecf@o386227.ingest.sentry.io/5911639",
)
ENABLE_SENTRY = envvar.get("ENABLE_SENTRY", "true") == "true"

VESSL_DATA_COLLECTION_OPT_OUT = (
    envvar.get("VESSL_DATA_COLLECTION_OPT_OUT", "false").lower() == "true"
)
DATADOG_HOST = "https://http-intake.logs.datadoghq.com/api/v2/logs"
DATADOG_CLIENT_TOKEN = "pubfcd304b1349d775f3c0ffffce7ff2283"

VESSL_HELM_REPO = "https://vessl-ai.github.io/helm-charts"
VESSL_HELM_CHART_NAME = "vessl"

K0S_VERSION = "v1.25.12-k0s.0"

DEFAULT_VESSL_SIDECAR_HOST = envvar.get("VESSL_SIDECAR_HOST", "http://localhost:3551")
UPDATE_CONTEXT_VARIABLE_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/store"
GET_ARGUMENT_VALUE_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/argument"
GET_CONTEXT_VARIABLE_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/context"
RESUME_STEP_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/workload/stop_idle"

LOGIN_TIMEOUT_SECONDS = 160

VESSL_JWT_DIR = "/opt/vessl/access_token"
JWT_PATH = "/opt/vessl/access_token/token_file"
ACCESS_TOKEN_ENV_VAR = "VESSL_ACCESS_TOKEN"
MODEL_SERVICE_JOIN_TOKEN_ENV_VAR = "VESSL_MODEL_SERVICE_JOIN_TOKEN"
DEFAULT_ORGANIZATION_ENV_VAR = "VESSL_DEFAULT_ORGANIZATION"
DEFAULT_PROJECT_ENV_VAR = "VESSL_DEFAULT_PROJECT"
CREDENTIALS_FILE_ENV_VAR = "VESSL_CREDENTIALS_FILE"
PROJECT_NAME_ENV_VAR = "VESSL_PROJECT_NAME"

CLUSTER_KUBECONFIG_ENV_VAR = "VESSL_CLUSTER_KUBECONFIG"
CLUSTER_MODE_SINGLE = "single"
CLUSTER_MODE_MULTI_NODE = "multi"

PROJECT_TYPE_VERSION_CONTROL = "version-control"
PROJECT_TYPES = [PROJECT_TYPE_VERSION_CONTROL]

DATASET_PATH_SCHEME_S3 = "s3://"
DATASET_PATH_SCHEME_GS = "gs://"

DATASET_VERSION_HASH_LATEST = "latest"

PROCESSOR_TYPE_CPU = "CPU"
PROCESSOR_TYPE_GPU = "GPU"
PROCESSOR_TYPES = [PROCESSOR_TYPE_CPU, PROCESSOR_TYPE_GPU]

SWEEP_OBJECTIVE_TYPE_MAXIMIZE = "maximize"
SWEEP_OBJECTIVE_TYPE_MINIMIZE = "minimize"
SWEEP_OBJECTIVE_TYPES = [SWEEP_OBJECTIVE_TYPE_MAXIMIZE, SWEEP_OBJECTIVE_TYPE_MINIMIZE]

SWEEP_ALGORITHM_TYPE_GRID = "grid"
SWEEP_ALGORITHM_TYPE_RANDOM = "random"
SWEEP_ALGORITHM_TYPE_BAYESIAN = "bayesian"
SWEEP_ALGORITHM_TYPES = [
    SWEEP_ALGORITHM_TYPE_GRID,
    SWEEP_ALGORITHM_TYPE_RANDOM,
    SWEEP_ALGORITHM_TYPE_BAYESIAN,
]

SWEEP_PARAMETER_TYPE_CATEGORICAL = "categorical"
SWEEP_PARAMETER_TYPE_INT = "int"
SWEEP_PARAMETER_TYPE_DOUBLE = "double"
SWEEP_PARAMETER_TYPES = [
    SWEEP_PARAMETER_TYPE_CATEGORICAL,
    SWEEP_PARAMETER_TYPE_INT,
    SWEEP_PARAMETER_TYPE_DOUBLE,
]

SWEEP_PARAMETER_RANGE_TYPE_SPACE = "space"
SWEEP_PARAMETER_RANGE_TYPE_LIST = "list"
SWEEP_PARAMETER_RANGE_TYPES = [
    SWEEP_PARAMETER_RANGE_TYPE_SPACE,
    SWEEP_PARAMETER_RANGE_TYPE_LIST,
]

MODEL_SOURCE_EXPERIMENT = "experiment"
MODEL_SOURCE_LOCAL = "local"

SOURCE_TYPE_CODE = "code"
SOURCE_TYPE_ARCHIVE_FILE = "archive-file"
SOURCE_TYPE_OBJECT_STORAGE = "object-storage"
SOURCE_TYPE_DATASET = "dataset"
SOURCE_TYPE_DATASET_VERSION = "dataset-version"
SOURCE_TYPE_MODEL_VOLUME = "model-volume"
SOURCE_TYPE_EMPTY_DIR = "empty-dir"
SOURCE_TYPE_OUTPUT = "output"
SOURCE_TYPE_PROJECT = "project"
SOURCE_TYPE_OUTPUT = "output"
SOURCE_TYPE_OBJECT_STORAGE = "object-storage"

MOUNT_PATH_EMPTY_DIR = "/root/"
MOUNT_PATH_OUTPUT = "/output/"
MOUNT_PATH_PROJECT = "/root/{}"

EXPERIMENT_WORKING_DIR = "/root/"

FRAMEWORK_TYPE_PYTORCH = "pytorch"
FRAMEWORK_TYPE_TENSORFLOW = "tensorflow"
FRAMEWORK_TYPES = (FRAMEWORK_TYPE_PYTORCH, FRAMEWORK_TYPE_TENSORFLOW)

PARALLEL_WORKERS = envvar.get("VESSL_PARALLEL_WORKERS", 20)

VESSL_MEDIA_PATH = "vessl-media"
VESSL_IMAGE_PATH = "images"
VESSL_AUDIO_PATH = "audio"
VESSL_PLOTS_FILETYPE_IMAGE = "image"
VESSL_PLOTS_FILETYPE_IMAGES = "images"
VESSL_PLOTS_FILETYPE_AUDIO = "audio"

WORKSPACE_BACKUP_MAX_SIZE = 15 * 1024 * 1024 * 1024
WORKSPACE_BACKUP_MAX_SIZE_FORMATTED = humanfriendly.format_size(
    WORKSPACE_BACKUP_MAX_SIZE, binary=True
)

TEMP_DIR = tempfile.gettempdir()

SSH_CONFIG_PATH = os.path.join(Path.home(), ".ssh", "config")
SSH_CONFIG_FORMAT = """Host {host}
    User root
    Hostname {hostname}
    Port {port}
    StrictHostKeyChecking accept-new
    CheckHostIP no
"""
SSH_PUBLIC_KEY_PATH = os.path.join(Path.home(), ".ssh", "id_ed25519.pub")
SSH_PRIVATE_KEY_PATH = os.path.join(Path.home(), ".ssh", "id_ed25519")

VESSL_IGNORE_FILE_PATH = os.path.join(os.getcwd(), ".vesslignore")

VESSL_SERVICE_TORCH_BASE_IMAGE_TEMPLATE = (
    "quay.io/vessl-ai/torch:{pytorch_version}-cuda{cuda_version}"
)
VESSL_SERVICE_TENSORFLOW_BASE_IMAGE_TEMPLATE = (
    "quay.io/vessl-ai/tensorflow:{tensorflow_version}-cuda{cuda_version}"
)

VOLUME_TYPE_SV = "sv"
VOLUME_TYPE_VOLUME = "volume"
VOLUME_TYPE_LOCAL = "local"


class colors:
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    GREY = "\033[38;5;244m"


LOGO = f"""{colors.OKCYAN}
        $$\    $$\ $$$$$$$$\  $$$$$$\   $$$$$$\  $$\              $$$$$$\  $$$$$$\
        $$ |   $$ |$$  _____|$$  __$$\ $$  __$$\ $$ |            $$  __$$\ \_$$  _|
        $$ |   $$ |$$ |      $$ /  \__|$$ /  \__|$$ |            $$ /  $$ |  $$ |
        \$$\  $$  |$$$$$\    \$$$$$$\  \$$$$$$\  $$ |            $$$$$$$$ |  $$ |
         \$$\$$  / $$  __|    \____$$\  \____$$\ $$ |            $$  __$$ |  $$ |
          \$$$  /  $$ |      $$\   $$ |$$\   $$ |$$ |            $$ |  $$ |  $$ |
           \$  /   $$$$$$$$\ \$$$$$$  |\$$$$$$  |$$$$$$$$\       $$ |  $$ |$$$$$$\
            \_/    \________| \______/  \______/ \________|      \__|  \__|\______|
        {colors.OKCYAN}"""

TRACING_AGREEMENT = f"""# Tracing Agreement
Purpose of Data Collection: We are committed to improving our CLI tool and enhancing user experience. To achieve this, we collect data on user interactions with our CLI commands.
This data helps us understand how our tool is used, identify areas for improvement, and ensure the reliability and performance of our services.

Data Collected
1. Executed Command Types: commands executed by users to understand usage patterns and identify commonly used features (this does not include user provided options nor keys of any form)
2. System Version: system version information to ensure compatibility and provide targeted support.

Usage of Collected Data: The collected data will be used solely for the following purposes:
- To analyze and improve the functionality of our CLI tool.
- To identify and resolve bugs or issues.
- To provide better support and updates based on user needs and system compatibility.

Data Storage and Privacy: The collected data is stored securely and access is restricted to authorized personnel only.
We do not share the collected data with third parties, except as required by law.

User Consent: By using our CLI tool, you consent to the collection and use of the above data as described in this agreement.
If you do not agree to this data collection, you can choose to opt out by setting `export VESSL_DATA_COLLECTION_OPT_OUT=true`.

Contact Information: If you have any questions or concerns about this agreement or the collection process, please contact us at: support@vessl.ai

Thank you for helping us improve our services.
_Last updated: 2024.07.10_
"""

MISTRAL_7B = """
name: mistral-7b-streamlit
description: A template Run for inference of Mistral-7B with streamlit app
resources:
  cluster: vessl-gcp-oregon
  preset: gpu-l4-small
image: quay.io/vessl-ai/hub:torch2.1.0-cuda12.2-202312070053
import:
  /model/: hf://huggingface.co/VESSL/Mistral-7B
  /code/:
    git:
      url: https://github.com/vessl-ai/hub-model
      ref: main
run:
  - command: |-
      pip install -r requirements_streamlit.txt
      streamlit run streamlit_demo.py --server.port 80
    workdir: /code/mistral-7B
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - name: streamlit
    type: http
    port: 80
"""

SSD_1B = """
name: SSD-1B-streamlit
description: A template Run for inference of SSD-1B with streamlit app
resources:
  cluster: vessl-gcp-oregon
  preset: gpu-l4-small
image: quay.io/vessl-ai/hub:torch2.1.0-cuda12.2-202312070053
import:
  /code/:
    git:
      url: https://github.com/vessl-ai/hub-model
      ref: main
  /model/: hf://huggingface.co/VESSL/SSD-1B
run:
  - command: |-
      pip install --upgrade pip
      pip install -r requirements.txt
      pip install git+https://github.com/huggingface/diffusers
      streamlit run ssd_1b_streamlit.py --server.port=80
    workdir: /code/SSD-1B
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - name: streamlit
    type: http
    port: 80
"""

WHISPER_V3 = """
name: whisper-v3
description: A template Run for inference of whisper v3 on librispeech_asr test set
resources:
  cluster: vessl-gcp-oregon
  preset: gpu-l4-small
image: quay.io/vessl-ai/hub:torch2.1.0-cuda12.2-202312070053
import:
  /model/: hf://huggingface.co/VESSL/Whisper-large-v3
  /dataset/: hf://huggingface.co/datasets/VESSL/librispeech_asr_clean_test
  /code/:
    git:
      url: https://github.com/vessl-ai/hub-model
      ref: main
run:
  - command: |-
      pip install -r requirements.txt
      python inference.py
    workdir: /code/whisper-v3
"""

VEGAART = """
name: segmind-vegart
description: A template Run for inference of Segmind VegaRT with streamlit app
resources:
  cluster: vessl-gcp-oregon
  preset: gpu-l4-small
image: quay.io/vessl-ai/hub:torch2.1.0-cuda12.2-202312070053
import:
  /code/:
    git:
      url: https://github.com/vessl-ai/hub-model
      ref: main
  /model/vega/: hf://huggingface.co/segmind/Segmind-Vega
  /model/vegart/: hf://huggingface.co/segmind/Segmind-VegaRT
run:
  - command: |-
      pip install --upgrade pip
      pip install -r requirements.txt
      streamlit run app_rt.py --server.port=80
    workdir: /code/segmind-vega
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - name: streamlit
    type: http
    port: 80
"""

JUPYTER = """
name: gpu-interactive-run
description: Run an interactive GPU-backed Jupyter and SSH server.
tags:
  - interactive
  - jupyter
  - ssh
  - v100-1
resources:
  cluster: vessl-gcp-oregon
  preset: gpu-l4-small
image: quay.io/vessl-ai/ngc-pytorch-kernel:22.10-py3-202306140422
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
"""

LLAMA_2_PLAYGROUND = """
name: llama2_c_playground
description: Batch inference with llama2.c.
tags:
  - playground
  - inference
resources:
  cluster: vessl-aws-seoul
  preset: v1.cpu-2.mem-6
image: quay.io/vessl-ai/ngc-pytorch-kernel:23.07-py3-202308010607
import:
  /root/examples/: git://github.com/vessl-ai/examples.git
run:
  - command: pip install streamlit
    workdir: /root/examples
  - command: wget https://huggingface.co/karpathy/tinyllamas/resolve/main/stories42M.bin
    workdir: /root/examples/llama2_c
  - command: streamlit run llama2_c/streamlit/llama2_c_inference.py --server.port=80
    workdir: /root/examples
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - name: streamlit
    type: http
    port: 80
"""
LLAMA_2_TRAIN = """
name: llama2_c_training
description: Training llama2_c from scratch with VESSL Run.
tags:
  - training
resources:
  cluster: vessl-aws-seoul
  preset: v1.a10g-1.mem-26
image: quay.io/vessl-ai/ngc-pytorch-kernel:23.07-py3-202308010607
import:
  /root/llama2_c/: git://github.com/karpathy/llama2.c.git
export:
  /output/: vessl-artifact://
run:
  - command: |-
      pip install -r requirements.txt && \
      python tinystories.py download && \
      python tinystories.py pretokenize && \
      python train.py --dtype=float16 --compile=False --eval_iters=$eval_iters --batch_size=$batch_size --max_iters=$max_iters --out_dir=$out_dir
    workdir: /root/llama2_c
ports: []
env:
  batch_size: "8"
  dtype: float16
  eval_iters: "10"
  max_iters: "10000"
  out_dir: /output
service_account_name: ""
termination_protection: false
"""
LLAMA_2_FINE_TUNING = """
name: train_tolkien_llama_2
resources:
  cluster: vessl-aws-seoul
  preset: v1.v100-1.mem-52
image: quay.io/vessl-ai/ngc-pytorch-kernel:23.09-py3-202310150329
import:
  /ckpt/: vessl-model://vessl-ai/llama2/1
  /root/examples/: git://github.com/vessl-ai/examples.git
run:
  - command: |-
      pip install --upgrade pip
      pip install transformers datasets accelerate peft bitsandbytes sentencepiece -qq
      wget https://raw.githubusercontent.com/jeremyarancio/llm-tolkien/main/llm/data/extracted_text.jsonl
      cd /ckpt/
      unzip llama_2_7b_hf.zip
      cd /root/examples/LLM_llama_2_LOTR
      python train_tolkien_llama_2.py
    workdir: /root/examples/LLM_llama_2_LOTR
"""

_print_envvar_related_constants_on_dev()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ProtoLLMWorkloadConnectionConfiguration(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'knowledge_permission_configs': 'dict[str, V1LLMPermissionConfig]',
        'model_connection_configs': 'dict[str, ProtoLLMModelConnection]',
        'model_secrets': 'dict[str, ProtoRevealedSecret]',
        'vector_db_collections': 'dict[str, str]',
        'vector_db_connection_configs': 'dict[str, ProtoLLMVectorDBConnConfig]',
        'workflow_permission_config': 'V1LLMPermissionConfig'
    }

    attribute_map = {
        'knowledge_permission_configs': 'knowledge_permission_configs',
        'model_connection_configs': 'model_connection_configs',
        'model_secrets': 'model_secrets',
        'vector_db_collections': 'vector_db_collections',
        'vector_db_connection_configs': 'vector_db_connection_configs',
        'workflow_permission_config': 'workflow_permission_config'
    }

    def __init__(self, knowledge_permission_configs=None, model_connection_configs=None, model_secrets=None, vector_db_collections=None, vector_db_connection_configs=None, workflow_permission_config=None, local_vars_configuration=None):  # noqa: E501
        """ProtoLLMWorkloadConnectionConfiguration - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._knowledge_permission_configs = None
        self._model_connection_configs = None
        self._model_secrets = None
        self._vector_db_collections = None
        self._vector_db_connection_configs = None
        self._workflow_permission_config = None
        self.discriminator = None

        if knowledge_permission_configs is not None:
            self.knowledge_permission_configs = knowledge_permission_configs
        if model_connection_configs is not None:
            self.model_connection_configs = model_connection_configs
        if model_secrets is not None:
            self.model_secrets = model_secrets
        if vector_db_collections is not None:
            self.vector_db_collections = vector_db_collections
        if vector_db_connection_configs is not None:
            self.vector_db_connection_configs = vector_db_connection_configs
        if workflow_permission_config is not None:
            self.workflow_permission_config = workflow_permission_config

    @property
    def knowledge_permission_configs(self):
        """Gets the knowledge_permission_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501


        :return: The knowledge_permission_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :rtype: dict[str, V1LLMPermissionConfig]
        """
        return self._knowledge_permission_configs

    @knowledge_permission_configs.setter
    def knowledge_permission_configs(self, knowledge_permission_configs):
        """Sets the knowledge_permission_configs of this ProtoLLMWorkloadConnectionConfiguration.


        :param knowledge_permission_configs: The knowledge_permission_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :type knowledge_permission_configs: dict[str, V1LLMPermissionConfig]
        """

        self._knowledge_permission_configs = knowledge_permission_configs

    @property
    def model_connection_configs(self):
        """Gets the model_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501


        :return: The model_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :rtype: dict[str, ProtoLLMModelConnection]
        """
        return self._model_connection_configs

    @model_connection_configs.setter
    def model_connection_configs(self, model_connection_configs):
        """Sets the model_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.


        :param model_connection_configs: The model_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :type model_connection_configs: dict[str, ProtoLLMModelConnection]
        """

        self._model_connection_configs = model_connection_configs

    @property
    def model_secrets(self):
        """Gets the model_secrets of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501


        :return: The model_secrets of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :rtype: dict[str, ProtoRevealedSecret]
        """
        return self._model_secrets

    @model_secrets.setter
    def model_secrets(self, model_secrets):
        """Sets the model_secrets of this ProtoLLMWorkloadConnectionConfiguration.


        :param model_secrets: The model_secrets of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :type model_secrets: dict[str, ProtoRevealedSecret]
        """

        self._model_secrets = model_secrets

    @property
    def vector_db_collections(self):
        """Gets the vector_db_collections of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501


        :return: The vector_db_collections of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :rtype: dict[str, str]
        """
        return self._vector_db_collections

    @vector_db_collections.setter
    def vector_db_collections(self, vector_db_collections):
        """Sets the vector_db_collections of this ProtoLLMWorkloadConnectionConfiguration.


        :param vector_db_collections: The vector_db_collections of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :type vector_db_collections: dict[str, str]
        """

        self._vector_db_collections = vector_db_collections

    @property
    def vector_db_connection_configs(self):
        """Gets the vector_db_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501


        :return: The vector_db_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :rtype: dict[str, ProtoLLMVectorDBConnConfig]
        """
        return self._vector_db_connection_configs

    @vector_db_connection_configs.setter
    def vector_db_connection_configs(self, vector_db_connection_configs):
        """Sets the vector_db_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.


        :param vector_db_connection_configs: The vector_db_connection_configs of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :type vector_db_connection_configs: dict[str, ProtoLLMVectorDBConnConfig]
        """

        self._vector_db_connection_configs = vector_db_connection_configs

    @property
    def workflow_permission_config(self):
        """Gets the workflow_permission_config of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501


        :return: The workflow_permission_config of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :rtype: V1LLMPermissionConfig
        """
        return self._workflow_permission_config

    @workflow_permission_config.setter
    def workflow_permission_config(self, workflow_permission_config):
        """Sets the workflow_permission_config of this ProtoLLMWorkloadConnectionConfiguration.


        :param workflow_permission_config: The workflow_permission_config of this ProtoLLMWorkloadConnectionConfiguration.  # noqa: E501
        :type workflow_permission_config: V1LLMPermissionConfig
        """

        self._workflow_permission_config = workflow_permission_config

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProtoLLMWorkloadConnectionConfiguration):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProtoLLMWorkloadConnectionConfiguration):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ResponseReducedPipelineStepExecution(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'end_dt': 'datetime',
        'id': 'int',
        'judgment_result': 'str',
        'status': 'str',
        'step_key': 'str',
        'step_spec_id': 'int',
        'step_type': 'str',
        'sub_pipeline_execution': 'ResponsePipelineExecution',
        'title': 'str'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'end_dt': 'end_dt',
        'id': 'id',
        'judgment_result': 'judgment_result',
        'status': 'status',
        'step_key': 'step_key',
        'step_spec_id': 'step_spec_id',
        'step_type': 'step_type',
        'sub_pipeline_execution': 'sub_pipeline_execution',
        'title': 'title'
    }

    def __init__(self, created_dt=None, end_dt=None, id=None, judgment_result=None, status=None, step_key=None, step_spec_id=None, step_type=None, sub_pipeline_execution=None, title=None, local_vars_configuration=None):  # noqa: E501
        """ResponseReducedPipelineStepExecution - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._end_dt = None
        self._id = None
        self._judgment_result = None
        self._status = None
        self._step_key = None
        self._step_spec_id = None
        self._step_type = None
        self._sub_pipeline_execution = None
        self._title = None
        self.discriminator = None

        self.created_dt = created_dt
        self.end_dt = end_dt
        self.id = id
        self.judgment_result = judgment_result
        if status is not None:
            self.status = status
        self.step_key = step_key
        self.step_spec_id = step_spec_id
        self.step_type = step_type
        if sub_pipeline_execution is not None:
            self.sub_pipeline_execution = sub_pipeline_execution
        self.title = title

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The created_dt of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseReducedPipelineStepExecution.


        :param created_dt: The created_dt of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def end_dt(self):
        """Gets the end_dt of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The end_dt of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._end_dt

    @end_dt.setter
    def end_dt(self, end_dt):
        """Sets the end_dt of this ResponseReducedPipelineStepExecution.


        :param end_dt: The end_dt of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type end_dt: datetime
        """

        self._end_dt = end_dt

    @property
    def id(self):
        """Gets the id of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The id of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseReducedPipelineStepExecution.


        :param id: The id of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def judgment_result(self):
        """Gets the judgment_result of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The judgment_result of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._judgment_result

    @judgment_result.setter
    def judgment_result(self, judgment_result):
        """Sets the judgment_result of this ResponseReducedPipelineStepExecution.


        :param judgment_result: The judgment_result of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type judgment_result: str
        """

        self._judgment_result = judgment_result

    @property
    def status(self):
        """Gets the status of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The status of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseReducedPipelineStepExecution.


        :param status: The status of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type status: str
        """

        self._status = status

    @property
    def step_key(self):
        """Gets the step_key of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The step_key of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._step_key

    @step_key.setter
    def step_key(self, step_key):
        """Sets the step_key of this ResponseReducedPipelineStepExecution.


        :param step_key: The step_key of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type step_key: str
        """
        if self.local_vars_configuration.client_side_validation and step_key is None:  # noqa: E501
            raise ValueError("Invalid value for `step_key`, must not be `None`")  # noqa: E501

        self._step_key = step_key

    @property
    def step_spec_id(self):
        """Gets the step_spec_id of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The step_spec_id of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: int
        """
        return self._step_spec_id

    @step_spec_id.setter
    def step_spec_id(self, step_spec_id):
        """Sets the step_spec_id of this ResponseReducedPipelineStepExecution.


        :param step_spec_id: The step_spec_id of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type step_spec_id: int
        """
        if self.local_vars_configuration.client_side_validation and step_spec_id is None:  # noqa: E501
            raise ValueError("Invalid value for `step_spec_id`, must not be `None`")  # noqa: E501

        self._step_spec_id = step_spec_id

    @property
    def step_type(self):
        """Gets the step_type of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The step_type of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._step_type

    @step_type.setter
    def step_type(self, step_type):
        """Sets the step_type of this ResponseReducedPipelineStepExecution.


        :param step_type: The step_type of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type step_type: str
        """
        if self.local_vars_configuration.client_side_validation and step_type is None:  # noqa: E501
            raise ValueError("Invalid value for `step_type`, must not be `None`")  # noqa: E501

        self._step_type = step_type

    @property
    def sub_pipeline_execution(self):
        """Gets the sub_pipeline_execution of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The sub_pipeline_execution of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineExecution
        """
        return self._sub_pipeline_execution

    @sub_pipeline_execution.setter
    def sub_pipeline_execution(self, sub_pipeline_execution):
        """Sets the sub_pipeline_execution of this ResponseReducedPipelineStepExecution.


        :param sub_pipeline_execution: The sub_pipeline_execution of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type sub_pipeline_execution: ResponsePipelineExecution
        """

        self._sub_pipeline_execution = sub_pipeline_execution

    @property
    def title(self):
        """Gets the title of this ResponseReducedPipelineStepExecution.  # noqa: E501


        :return: The title of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ResponseReducedPipelineStepExecution.


        :param title: The title of this ResponseReducedPipelineStepExecution.  # noqa: E501
        :type title: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseReducedPipelineStepExecution):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseReducedPipelineStepExecution):
            return True

        return self.to_dict() != other.to_dict()



from typing import Any, Dict, List, Tuple

import click

from vessl.openapi_client import OrmParameter, OrmSweepObjective
from vessl.cli._base import VesslGroup, vessl_argument, vessl_option
from vessl.cli._util import (
    print_data,
  print_logs,
  print_table,
  truncate_datetime,
)
from vessl.util.fmt import format_string
from vessl.util.prompt import prompt_choices, generic_prompter, choices_prompter
from vessl.util.echo import print_warning, print_info, print_success
from vessl.util.endpoint import Endpoint
from vessl.cli.experiment import (
    archive_file_option,
    cluster_option,
    command_option,
    cpu_limit_option,
    dataset_option,
    git_diff_option,
    git_ref_option,
    gpu_limit_option,
    gpu_type_option,
    hyperparameter_option,
    image_url_option,
    memory_limit_option,
    message_option,
    object_storage_option,
    output_dir_option,
    processor_type_option,
    resource_option,
    root_volume_size_option,
    working_dir_option,
)
from vessl.cli.organization import organization_name_option
from vessl.cli.project import project_name_option
from vessl.sweep import (
    create_sweep,
    get_best_sweep_experiment,
    list_sweep_logs,
    list_sweeps,
    read_sweep,
    terminate_sweep,
)
from vessl.util.constant import (
    SWEEP_ALGORITHM_TYPES,
    SWEEP_OBJECTIVE_TYPE_MAXIMIZE,
    SWEEP_OBJECTIVE_TYPES,
    SWEEP_PARAMETER_RANGE_TYPE_LIST,
    SWEEP_PARAMETER_RANGE_TYPE_SPACE,
    SWEEP_PARAMETER_RANGE_TYPES,
    SWEEP_PARAMETER_TYPES,
)


class SweepParameterType(click.ParamType):
    name = "Parameter type"

    def convert(self, raw_value: Any, param, ctx) -> Any:
        tokens = raw_value.split()

        if len(tokens) < 4:
            raise click.BadOptionUsage(
                option_name="parameter",
                message=f"Invalid value for [PARAMETER]: '{raw_value}' must be of form [name] [type] [range type] [values...].",
            )

        name = tokens[0]
        type = click.Choice(SWEEP_PARAMETER_TYPES).convert(tokens[1], param, ctx)
        range_type = click.Choice(SWEEP_PARAMETER_RANGE_TYPES).convert(tokens[2], param, ctx)
        values = tokens[3:]

        parameter = {"name": name, "type": type}

        if range_type == SWEEP_PARAMETER_RANGE_TYPE_LIST:
            parameter["range"] = {"list": values}
            return parameter

        if len(values) < 2:
            raise click.BadOptionUsage(
                option_name="parameter",
                message=f"Invalid value for [PARAMETER]: range type '{SWEEP_PARAMETER_RANGE_TYPE_SPACE}' must have min and max values.",
            )

        parameter["range"] = {
            "min": values[0],
            "max": values[1],
        }
        if len(values) >= 3:
            parameter["range"]["step"] = values[2]

        return parameter


def sweep_name_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    sweeps = list_sweeps()
    return prompt_choices("Sweep", [x.name for x in sweeps])


def parameter_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    parameters = []

    while True:
        index = len(parameters) + 1
        name = click.prompt(f"Parameter #{index} name")
        type = prompt_choices(f"Parameter #{index} type", SWEEP_PARAMETER_TYPES)
        range_type = prompt_choices(f"Parameter #{index} range type", SWEEP_PARAMETER_RANGE_TYPES)

        if range_type == SWEEP_PARAMETER_RANGE_TYPE_LIST:
            values = click.prompt(f"Parameter #{index} values (space separated)")
        else:
            values = click.prompt(f"Parameter #{index} values ([min] [max] [step])")

        parameters.append(f"{name} {type} {range_type} {values}")

        if not click.prompt("Add another parameter (y/n)", type=click.BOOL):
            break

    return parameters


@click.command(name="sweep", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument("name", type=click.STRING, required=True, prompter=sweep_name_prompter)
@organization_name_option
@project_name_option
def read(name: str):
    sweep = read_sweep(sweep_name=name)
    print_data(
        {
            "ID": sweep.id,
            "Name": sweep.name,
            "Status": sweep.status,
            "Created": truncate_datetime(sweep.created_dt),
            "Message": format_string(sweep.message),
            "Objective": (
                f"{sweep.objective.type}"
                f"{' > ' if sweep.objective.metric == SWEEP_OBJECTIVE_TYPE_MAXIMIZE else ' < '}"
                f"{sweep.objective.goal}"
            ),
            "Common Parameters": {
                "Max Experiment Count": sweep.max_experiment_count,
                "Parallel Experiment Count": sweep.parallel_experiment_count,
                "Max Failed Experiment Count": sweep.max_failed_experiment_count,
            },
            "Algorithm": sweep.algorithm,
            "Parameters": [
                {
                    "Name": x.name,
                    "Type": x.type,
                    "Values": {
                        "Min": x.range.min,
                        "Max": x.range.max,
                        "Step": x.range.step,
                    }
                    if x.range.list is None
                    else {
                        "List": x.range.list,
                    },
                }
                for x in sweep.search_space.parameters
            ],
            "Experiments": f"{sweep.experiment_summary.total}/{sweep.max_experiment_count}",
            "Kernel Image": {
                "Name": sweep.kernel_image.name,
                "URL": sweep.kernel_image.image_url,
            },
            "Resource Spec": {
                "Name": sweep.kernel_resource_spec.name,
                "CPU Type": sweep.kernel_resource_spec.cpu_type,
                "CPU Limit": sweep.kernel_resource_spec.cpu_limit,
                "Memory Limit": sweep.kernel_resource_spec.memory_limit,
                "GPU Type": sweep.kernel_resource_spec.gpu_type,
                "GPU Limit": sweep.kernel_resource_spec.gpu_limit,
            },
            "Start command": sweep.start_command,
        }
    )
    print_info(
        f"For more info: {Endpoint.sweep.format(sweep.organization.name, sweep.project.name, sweep.name)}"
    )


@cli.vessl_command()
@organization_name_option
@project_name_option
def list():
    sweeps = list_sweeps()
    print_table(
        sweeps,
        ["Name", "Status", "Created", "Experiments"],
        lambda x: [
            x.name,
            x.status,
            truncate_datetime(x.created_dt),
            f"{x.experiment_summary.total}/{x.max_experiment_count}",
        ],
    )


@cli.vessl_command()
@vessl_argument("name", type=click.STRING, prompter=generic_prompter("Name"))
@vessl_option(
    "-T",
    "--objective-type",
    type=click.Choice(SWEEP_OBJECTIVE_TYPES),
    prompter=choices_prompter("Objective type", SWEEP_OBJECTIVE_TYPES),
)
@vessl_option(
    "-M",
    "--objective-metric",
    type=click.STRING,
    prompter=generic_prompter("Objective metric", click.STRING),
)
@vessl_option(
    "-G",
    "--objective-goal",
    type=click.STRING,
    prompter=generic_prompter("Objective goal", click.FLOAT),
)
@vessl_option(
    "--num-experiments",
    type=click.INT,
    prompter=generic_prompter("Maximum number of experiments", click.INT),
    help="Maximum number of experiments.",
)
@vessl_option(
    "--num-parallel",
    type=click.INT,
    prompter=generic_prompter("Number of experiments to be run in parallel", click.INT),
    help="Number of experiments to be run in parallel.",
)
@vessl_option(
    "--num-failed",
    type=click.INT,
    prompter=generic_prompter("Maximum number of experiments to allow to fail", click.INT),
    help="Maximum number of experiments to allow to fail.",
)
@vessl_option(
    "-a",
    "--algorithm",
    type=click.Choice(SWEEP_ALGORITHM_TYPES),
    required=True,
    prompter=choices_prompter("Sweep algorithm", SWEEP_ALGORITHM_TYPES),
    help="Sweep algorithm.",
)
@vessl_option(
    "-p",
    "--parameter",
    type=SweepParameterType(),
    multiple=True,
    prompter=parameter_prompter,
    help="Search space parameters (at least one required). Format: [name] [type] [range type] [values...], ex. `-p epochs int space 5 10 15 20`.",
)
@cluster_option
@resource_option
@processor_type_option
@cpu_limit_option
@memory_limit_option
@gpu_type_option
@gpu_limit_option
@image_url_option
@hyperparameter_option
@command_option
@vessl_option("--early-stopping-name", type=str, help="Early stopping algorithm name.")
@vessl_option(
    "--early-stopping-settings",
    type=click.Tuple([str, str]),
    multiple=True,
    help="Early stopping algorithm settings. Format: [key] [value], ex. `--early-stopping-settings start_step 4`.",
)
@message_option
@dataset_option
@git_ref_option
@git_diff_option
@archive_file_option
@object_storage_option
@root_volume_size_option
@working_dir_option
@output_dir_option
@organization_name_option
@project_name_option
def create(
    name: str,
    objective_type: str,
    objective_goal: str,
    objective_metric: str,
    num_experiments: int,
    num_parallel: int,
    num_failed: int,
    algorithm: str,
    parameter: List[Dict[str, Any]],
    cluster: str,
    command: str,
    resource: str,
    processor_type: str,
    cpu_limit: float,
    memory_limit: str,
    gpu_type: str,
    gpu_limit: int,
    image_url: str,
    early_stopping_name: str,
    early_stopping_settings: List[Tuple[str, str]],
    message: str,
    hyperparameter: List[Tuple[str, str]],
    dataset: List[str],
    git_ref: List[str],
    git_diff: str,
    archive_file: str,
    root_volume_size: str,
    working_dir: str,
    output_dir: str,
):
    objective = None
    if objective_type is not None:
        objective = OrmSweepObjective(
            type=objective_type,
            goal=objective_goal,
            metric=objective_metric,
        )
    sweep = create_sweep(
        name=name,
        objective=objective,
        max_experiment_count=num_experiments,
        parallel_experiment_count=num_parallel,
        max_failed_experiment_count=num_failed,
        algorithm=algorithm,
        parameters=[
            OrmParameter(name=p.get("name"), type=p.get("type"), range=p.get("range"))
            for p in parameter
        ],
        cluster_name=cluster,
        command=command,
        resource_spec_name=resource,
        processor_type=processor_type,
        cpu_limit=cpu_limit,
        memory_limit=memory_limit,
        gpu_type=gpu_type,
        gpu_limit=gpu_limit,
        image_url=image_url,
        early_stopping_name=early_stopping_name,
        early_stopping_settings=early_stopping_settings,
        message=message,
        hyperparameters=hyperparameter,
        dataset_mounts=dataset,
        git_ref_mounts=git_ref,
        git_diff_mount=git_diff,
        archive_file_mount=archive_file,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
    )
    print_success(
        f"Created '{sweep.name}'.\n"
        f"For more info: {Endpoint.sweep.format(sweep.organization.name, sweep.project.name, sweep.name)}"
    )


@cli.vessl_command()
@vessl_argument("name", type=click.STRING, required=True, prompter=sweep_name_prompter)
@organization_name_option
@project_name_option
def terminate(name: str):
    sweep = terminate_sweep(sweep_name=name)
    print_success(
        f"Terminated '{sweep.name}'.\n"
        f"For more info: {Endpoint.sweep.format(sweep.organization.name, sweep.project.name, sweep.name)}"
    )


@cli.vessl_command()
@vessl_argument("name", type=click.STRING, required=True, prompter=sweep_name_prompter)
@click.option(
    "--tail",
    type=click.INT,
    default=200,
    help="Number of lines to display (from the end).",
)
@organization_name_option
@project_name_option
def logs(name: str, tail: int):
    logs = list_sweep_logs(sweep_name=name, limit=tail)
    print_logs(logs)
    print_info(f"Displayed last {len(logs)} lines of '{name}'.")


@cli.vessl_command()
@vessl_argument("name", type=click.STRING, required=True, prompter=sweep_name_prompter)
@organization_name_option
@project_name_option
def best_experiment(name: str):
    experiment = get_best_sweep_experiment(sweep_name=name)
    if not experiment.initialized_dt:
        print_warning("No experiments in sweep.")
        return

    print_data(
        {
            "Experiment number": experiment.experiment_number,
            "Objective": {
                "Metric name": experiment.objective_metric_name,
                "Metric value": experiment.objective_metric_value,
            },
            "Parameters": [
                {
                    "Name": x.name,
                    "Value": x.value,
                }
                for x in experiment.parameters
            ],
        }
    )



from typing import List

from vessl.openapi_client.models import ProjectCreateAPIInput, ResponseProjectInfo
from vessl import vessl_api
from vessl.organization import _get_organization_name
from vessl.util.exception import InvalidProjectError


def read_project(project_name: str, **kwargs) -> ResponseProjectInfo:
    """Read project in the default organization. If you want to
    override the default organization, then pass `organization_name` as
    `**kwargs`.

    Args:
        project_name(str): Project name.

    Example:
        ```python
        vessl.read_project(
            project_name="tutorials",
        )
        ```
    """
    return vessl_api.project_read_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=project_name,
    )


def list_projects(**kwargs) -> List[ResponseProjectInfo]:
    """List projects in the default organization. If you want to
    override the default organization, then pass `organization_name` as
    `**kwargs`.

    Example:
        ```python
        vessl.list_projects()
        ```
    """

    query_keys = set(["limit", "offset"])
    query_kwargs = {k: v for k, v in kwargs.items() if k in query_keys}

    return vessl_api.project_list_api(
        organization_name=_get_organization_name(**kwargs), **query_kwargs
    ).results


def create_project(
    project_name: str,
    description: str = None,
    **kwargs,
) -> ResponseProjectInfo:
    """Create project in the default organization. If you want to
    override the default organization, then pass `organization_name` as
    `**kwargs`.

    Args:
        project_name(str): Project name.
        description(str): Project description. Defaults to None.

    Example:
        ```python
        vessl.create_project(
            project_name="tutorials",
            description="VESSL tutorial project",
        )
        ```
    """
    return vessl_api.project_create_api(
        organization_name=_get_organization_name(**kwargs),
        project_create_api_input=ProjectCreateAPIInput(
            name=project_name,
            description=description,
            project_repositories=[],  # TODO: support project repositories
        ),
    )

def delete_project(
    project_name: str,
    **kwargs,
) -> None:
    """Delete project in the default organization. If you want to
    override the default organization, then pass `organization_name` as
    `**kwargs`.

    Args:
        project_name(str): Project name.

    Example:
        ```python
        vessl.delete_project(
            project_name="no-longer-used",
        )
        ```
    """
    vessl_api.project_delete_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=project_name,
    )

### Deprecate for now
# def clone_project(project_name: str, **kwargs) -> None:
#     """Clone project

#     Keyword args:
#         organization_name (str): override default organization
#     """
#     project = read_project(project_name, **kwargs)

#     if project.type != PROJECT_TYPE_VERSION_CONTROL:
#         raise InvalidProjectError("Project type must be version-control.")

#     github_token = vessl_api.cli_git_hub_token_api().token

#     dirname = project.cached_git_repo_slug
#     git_url = f"https://{github_token}@github.com/{project.cached_git_owner_slug}/{project.cached_git_repo_slug}.git"
#     try:
#         subprocess.check_output(["git", "clone", git_url])
#     except subprocess.CalledProcessError:
#         dirname = f"savvihub-{project.cached_git_repo_slug}"
#         logger.info(f"Falling back to '{dirname}'...")
#         subprocess.check_output(["git", "clone", git_url, dirname])

#     subprocess.check_output(["rm", "-rf", f"{dirname}/.git"])


def _get_project_name(**kwargs) -> str:
    project_name = kwargs.get("project_name")
    if project_name is not None:
        return project_name
    if vessl_api.project is not None:
        return vessl_api.project.name
    raise InvalidProjectError("No project selected.")


def _get_project(**kwargs) -> ResponseProjectInfo:
    project_name = kwargs.get("project_name")
    if project_name is not None:
        return read_project(project_name)
    if vessl_api.project is not None:
        return vessl_api.project
    raise InvalidProjectError("No project selected.")



from typing import Optional

from vessl.openapi_client import V1LLMPermissionConfig, ProtoLLMKnowledge


class Knowledge(object):
    id: int
    name: str
    description: str
    embedding_model_id: int
    vector_db_collection_name: str
    vector_db_connection_id: int
    permission_config: Optional[V1LLMPermissionConfig]
    created_by_id: int
    cluster_id: int
    resource_spec_id: int

    def __init__(self, knowledge: ProtoLLMKnowledge):
        self.id = knowledge.id
        self.name = knowledge.name
        self.description = knowledge.description
        self.embedding_model_id = knowledge.embedding_model_id
        self.vector_db_collection_name = knowledge.vectordb_collection_name
        self.vector_db_connection_id = knowledge.vectordb_connection_id
        self.permission_config = knowledge.permission_config
        self.created_by_id = knowledge.created_by_id
        self.cluster_id = knowledge.cluster_id
        self.resource_spec_id = knowledge.resource_spec_id



from enum import Enum
from typing import Optional

from vessl.openapi_client import (
    ResponseKernelClusterNodeInfo,
    StorageCreateAPIInput,
    StorageValidateConnectionAPIInput,
    V1ClusterHostPathConfig,
    V1ClusterNfsConfig,
    V1Config,
    V1GcsConfig,
    V1S3Config,
)
from vessl import list_clusters, vessl_api
from vessl.organization import _get_organization_name
from vessl.util.exception import (
    NotFoundError,
    StorageConnectionError,
    VesslApiException,
)


class StorageType(Enum):
    GCS = "gcs"
    S3 = "s3"
    NFS = "cluster-nfs"
    HOST_PATH = "cluster-host-path"


def _separate_bucket_path(path: str):
    parts = path.split("/", 1)
    if len(parts) == 1:
        return path, "/"
    return parts


def list_storages(**kwargs):
    """List storages in the default organization. If you want to
    override the default organization, then pass `organization_name` as
    `**kwargs`.

    Example:
        ```python
        vessl.list_storages()
        ```
    """
    query_keys = {"limit", "offset"}
    query_kwargs = {k: v for k, v in kwargs.items() if k in query_keys}

    storage_response = vessl_api.storage_list_api(
        organization_name=_get_organization_name(**kwargs), **query_kwargs
    )

    return [storage_response.managed] + storage_response.external.results


def _get_storage_config(storage_type: StorageType, path: str):
    config = V1Config()

    if storage_type == StorageType.S3:
        bucket, base_path = _separate_bucket_path(path)
        config.s3_config = V1S3Config(
            base_path=base_path,
            bucket_name=bucket,
        )
        return config
    if storage_type == StorageType.GCS:
        bucket, base_path = _separate_bucket_path(path)
        config.gcs_config = V1GcsConfig(
            base_path=base_path,
            bucket_name=bucket,
        )
        return config
    if storage_type == StorageType.NFS:
        host, base_path = path.split(":", 1)
        config.cluster_nfs_config = V1ClusterNfsConfig(
            host=host,
            base_path=base_path,
        )
        return config
    if storage_type == StorageType.HOST_PATH:
        config.cluster_host_path_config = V1ClusterHostPathConfig(
            base_path=path,
        )
        return config


def _test_connection(storage_type: StorageType, path: str, organization_credential_name: str):
    if storage_type in {StorageType.NFS, StorageType.HOST_PATH}:
        return

    try:
        vessl_api.storage_validate_connection_api(
            organization_name=vessl_api.organization.name,
            storage_validate_connection_api_input=StorageValidateConnectionAPIInput(
                organization_credential_name=organization_credential_name,
                storage_type=storage_type.value,
                storage_config=_get_storage_config(storage_type, path),
            ),
        )
    except VesslApiException as e:
        raise StorageConnectionError(
            "Storage connection failed. Please check your bucket status and policy."
        )


def _get_cluster_id(storage_type: StorageType, cluster_name):
    if not storage_type in {StorageType.NFS, StorageType.HOST_PATH}:
        return None

    cluster_list = list_clusters()

    for cluster in cluster_list:
        cluster: ResponseKernelClusterNodeInfo
        if cluster.name == cluster_name:
            return cluster.id

    raise NotFoundError("Cluster name not found")


def create_storage(
    name: str,
    storage_type: StorageType,
    path: str,
    credential_name: Optional[str] = None,
    cluster_name: Optional[str] = None,
):
    """Create storage.

    Args:
        name(str): The name of the storage to create.
        storage_type(StorageType): The type of storage to create. (GCS|S3|CLUSTER_NFS|CLUSTER_HOST_PATH)
        path(str): The path of the storage.
                    - For `s3` : Path must in `{bucket_name}/{path}` (e.g. my-bucket)
                    - For `gcs` : Path must in `{bucket_name}/{path}` (e.g. my-bucket)
                    - For `nfs` : Path must be in `{server}:{path}` format (e.g. 192.168.1.100:/shared/data)
                    - For `host-path` : Path must be an absolute local path (e.g. /data/host-folder)
        credential_name(str): The name of the credential to use. Required if `storage_type` is S3 or GCS
        cluster_name(str): The name of the cluster to use. Required if `storage_type` is NFS or HostPath

    Example:
        ```python
        vessl.create_storage(
            name="my-storage",
            storage_type=vessl.StorageType.S3,
            path="my-bucket",
            credential_name="my-credential",
        )
        ```
    """
    if storage_type in {StorageType.GCS, StorageType.S3} and not credential_name:
        raise NotFoundError("You must provide a credential name when storage type is S3 or GCS")
    if (
        storage_type in {StorageType.NFS, StorageType.HOST_PATH}
        and not cluster_name
    ):
        raise NotFoundError(
            "You must provide a cluster name when storage type is ClusterNFS or ClusterHostPath"
        )

    _test_connection(storage_type, path, credential_name)

    vessl_api.storage_create_api(
        organization_name=vessl_api.organization.name,
        storage_create_api_input=StorageCreateAPIInput(
            name=name,
            cluster_id=_get_cluster_id(storage_type, cluster_name),
            organization_credential_name=credential_name,
            storage_config=_get_storage_config(storage_type, path),
            storage_type=storage_type.value,
        ),
    )


def delete_storage(name: str):
    """Delete storage.

    Args:
        name(str): The name of the storage to delete.

    Example:
        ```python
        vessl.delete_storage(name="my-storage")
        ```
    """
    vessl_api.storage_delete_api(
        organization_name=vessl_api.organization.name,
        storage_name=name,
    )
